/**
 * yeoman项目构建逻辑
 * @constructor generators
 *
 * @markliao
 */

var path = require('path');
var fs = require('fs');
var chalk = require('chalk');
var util = require('util');
var generators = require('yeoman-generator');
var yosay = require('yosay');
var path = require('path');

module.exports = class extends generators {
  constructor(args, opts) {
    super(args, opts);
    this.appName = path.basename(process.cwd());
    this.appAuthor = "wulimark";
  }

  initializing() {
    this.log(chalk.green('react-ym移动端项目开始构建!'));
  }

  prompting() {
    var questions = [{
        type: 'input',
        name: 'webName',
        message: '输入项目名称',
        default: this.appName
      }, {
        type: 'input',
        name: 'webVersion',
        message: '项目版本号',
        default: '1.0.0'
      }, {
      name: 'webServer',
      type: 'list',
      message: '请选择静态服务:',
      choices: [{
          name: '配置Nginx服务',
          value: true,
          checked: true
        },{
          name: '配置Node服务',
          value: false
    }]}];
    return this.prompt(questions).then(
      function(answers){
          for(var item in answers){
              answers.hasOwnProperty(item) && (this[item] = answers[item]);
          }
      }.bind(this));
  }

  configuring() {
    let defaultSettings = this.fs.readJSON(this.templatePath('package.json'));
    let packageSettings = {
      name: this.webName,
      private: true,
      version: this.webVersion,
      description: `${this.webName} - Generated by generator-react-ym`,
      main: 'index.js',
      scripts: defaultSettings.scripts,
      repository: defaultSettings.repository,
      keywords: [],
      author: this.appAuthor,
      devDependencies: defaultSettings.devDependencies,
      dependencies: defaultSettings.dependencies,
      jest: defaultSettings.jest,
      babel: defaultSettings.babel,
      eslintConfig: defaultSettings.eslintConfig,
      proxy: defaultSettings.proxy
    };
    this.fs.writeJSON(this.destinationPath('package.json'), packageSettings);
  }

  writing() {
    this.fs.copy(
      this.templatePath('config'),
      this.destinationPath('config')
    );
    this.fs.copy(
      this.templatePath('public'),
      this.destinationPath('public')
    );
    this.fs.copy(
      this.templatePath('scripts'),
      this.destinationPath('scripts')
    );
    this.fs.copy(
      this.templatePath('src'),
      this.destinationPath('src')
    );
    this.fs.copy(
      this.templatePath('package.json'),
      this.destinationPath('package.json')
    );
    if (this.webServer) {
      this.fs.copy(
        this.templatePath('nginx.conf'),
        this.destinationPath('nginx.conf')
      );
    } else {
      this.fs.copy(
        this.templatePath('server'),
        this.destinationPath('server')
      );
    }
  }

  install() {
    var that = this;
    this.installDependencies({
      bower: false,
      npm: true,
      callback: function () {
        that.log(chalk.green('npm依赖安装完成!'));
      }
    });
  }
  
  end() {
    console.log('react-ym移动端项目构建完成!');
    console.log( `　　　　　　　 　/　　 /　　　　　　　ヽ　　　　　　ヽ　　　　　ヽ: . : l
    　　　　　 　　 / /　/　　　 /　　　　　､　　　　　　ヽ　　　　　l: . . |
    　　　　　 　 /イ 　 l　　　/ |　　　　　　､　　　　　　l　　|　l 　 l: . .|
    　　　　　　　　| :.　.:| 　 /　 l　　　　　 　､　　　　　 :l　　l: :|＼ﾊ: ﾊ
    　　　　　　 　 ﾚ|: 　|　/　￣ |　　　 　　　!:.　　　　　l: : : ト| ー } |
    　　　　　　　 　 |: . l　| '￣ﾋぅ| 　 l　　　　:.:..　　　l:　 .:| /ﾚ　ｊ: / {
    　　　　　　　　　VヽN　\` ー　ｊﾊノ ＼|:　 :.:.|:.　l:.:.ﾊ:.:./ﾚ ﾉ ノ /. :ﾄ
    　　　　　　　 　 　 　 |　 　 　l　　　　 ヽ/ソﾄﾊﾚ　 ﾝ　　 ´ ノ〃 |
    　　　　　　　　　　　　l　　　 l　　　　　　　　　　　　　　　fl : ..　 |
    　　　　　　　　　　　　 l　　　ヽ　 _　　　　　　　　　　　　l　|ﾊ/|ﾉ＿
    　　　　　　　　　　　　　､　　　　　　　　　　　　　　　　 　　　　|　7 |
    　　　　　　　　　　　　　 ヽ　　　 　　　　　_　　　　　 /　　　　 ／　 l
    　　　　　　　　　　,.. -─‐‐-､ヽ二二二ノ　　　　／　　　 ／
    　　　　　　 ＿_L´-､　ノ　　　　　｀丶､　　　　,　'´ 　 　 ／
    　　　　 /´　　{_ 　.::}　　　　　. : . : . : .|_,.. - ´　　　　／\n\n\n\n
    没有想到吧,就是这么容易啊!!!`);
    console.log('给力的点个赞(^-^)!! https://github.com/wulimark/gennerator-react-ym')
  }
};