/**
 * 开发环境代码检测、编译、转换、热加载配置
 * @markliao
 */
var path = require('path');
var autoprefixer = require('autoprefixer');
var postcsspxtorem = require('postcss-pxtorem');
var webpack = require('webpack');
var findCacheDir = require('find-cache-dir');
var HtmlWebpackPlugin = require('html-webpack-plugin');
var CaseSensitivePathsPlugin = require('case-sensitive-paths-webpack-plugin');
var InterpolateHtmlPlugin = require('react-dev-utils/InterpolateHtmlPlugin');
var WatchMissingNodeModulesPlugin = require('react-dev-utils/WatchMissingNodeModulesPlugin');
var getClientEnvironment = require('./env');
var paths = require('./paths');

// 设置根路径
var publicPath = '/';
// 设置 %PUBLIC_URL% in `index.html` and `process.env.PUBLIC_URL` in JavaScript
var publicUrl = '';
// 获取环境变量
var env = getClientEnvironment(publicUrl);

// webpack development configuration.
module.exports = {
  // 可以使用'cheap-module-source-map'模块来替代source maps功能
  devtool: 'eval',
  // 模块加载入口
  entry: [
    // 代码修改热更新、错误提示
    require.resolve('react-dev-utils/webpackHotDevClient'),
    // 兼容性代码
    require.resolve('./polyfills'),
    // 模块入口
    paths.appIndexJs
  ],
  output: {
    // 编译入口
    path: paths.appBuild,
    // /* filename */注释
    pathinfo: true,
    // 编译生成文件名
    filename: 'static/js/bundle.js',
    // 静态路径
    publicPath: publicPath
  },
  resolve: {
    // 操作系统NODE路径设置
    fallback: paths.nodePaths,
    // 省略文件名后缀
    extensions: ['.js', '.json', '.jsx', '.scss', ''],
    alias: {
      // 模块引用简略
      'react-native': 'react-native-web'
    }
  },
  
  module: {
    // 代码检测
    preLoaders: [
      {
        test: /\.(js|jsx)$/,
        loader: 'eslint',
        include: paths.appSrc,
      }
    ],
    loaders: [
      // Babel转换
      {
        test: /\.(js|jsx)$/,
        include: paths.appSrc,
        loader: 'babel',
        query: {
          // webpack编译缓存使用
          cacheDirectory: findCacheDir({
            name: 'react-scripts'
          })
        }
      },
      // postcss、css转换
      {
        test: /\.scss$/,
        loader: 'style!css!sass!postcss'
      },
      // JSON模块支持
      {
        test: /\.json$/,
        loader: 'json'
      },
      // 静态文件引用
      {
        test: /\.(ico|gif|eot|otf|webp|svg|ttf|woff|woff2)(\?.*)?$/,
        loader: 'file',
        query: {
          name: 'static/media/[name].[hash:8].[ext]'
        }
      },
      // 小于10kb转换base64
      {
        test: /\.(jpg|jpeg|png|mp4|webm|wav|mp3|m4a|aac|oga)(\?.*)?$/,
        loader: 'url',
        query: {
          limit: 1000,
          name: 'static/images/[name].[hash:8].[ext]'
        }
      }
    ]
  },
  // 使用PostCSS for autoprefixing only
  postcss: function() {
    return [
      autoprefixer({
        browsers: [
          '>1%',
          'last 4 versions',
          'Firefox ESR',
          'not ie < 9', // React doesn't support IE8 anyway
        ]
      }),
      postcsspxtorem({
        rootValue: 37.5,
        unitPrecision: 5,
        propList: ['*'],
        selectorBlackList: [],
        replace: true,
        mediaQuery: true,
        minPixelValue: 0
      })
    ];
  },
  plugins: [
    // %PUBLIC_URL% in index.html
    // <link rel="shortcut icon" href="%PUBLIC_URL%/favicon.ico">
    new InterpolateHtmlPlugin({
      PUBLIC_URL: publicUrl
    }),
    new HtmlWebpackPlugin({
      inject: true,
      template: paths.appHtml,
    }),
    // 默认环境变量设置
    new webpack.DefinePlugin(env),
    // 热替换
    new webpack.HotModuleReplacementPlugin(),
    // 路径报错
    new CaseSensitivePathsPlugin(),
    // 模块缺失
    new WatchMissingNodeModulesPlugin(paths.appNodeModules)
  ],
  // 禁用模块
  node: {
    fs: 'empty',
    net: 'empty',
    tls: 'empty'
  },
  devServer: {
    // 端口映射
    proxy: {
      '/news-api': {
        target: 'http://app.xinrongnews.com',
        secure: false,
      }
    }
  }
};