/**
 * 开发Store模块
 * @markliao
 */
import {createStore, applyMiddleware, compose} from 'redux';
import thunk from 'redux-thunk';
import createLogger from 'redux-logger';
import rootReducer from '../reducers';
import DevTools from '../containers/DevTools';
import {persistStore, autoRehydrate} from 'redux-persist';
import {LocalStorage} from 'redux-persist/storages';
import {callAPIMiddleware, timeoutScheduler} from '../middleware/api';

const configureStore = preloadedState => {
  const store = createStore(
    rootReducer,
    preloadedState,
    compose(
      applyMiddleware(thunk, createLogger(), callAPIMiddleware, timeoutScheduler),
      DevTools.instrument(),
      autoRehydrate()
    )
  );
  if (module.hot) {
    module.hot.accept('../reducers', () => {
      const nextRootReducer = require('../reducers').default;
      store.replaceReducer(nextRootReducer);
    });
  }
  persistStore(store, {storage: LocalStorage});
  return store;
};

export default configureStore;